@@ -1,7 +1,7 @@
-id = "9905e10f24f54fe4f9633c6ef0d3c2b8724d8482"
-date = "2013-12-03 21:26:44 +0000"
+id = "45c9dd9d16bc677a761b629ab559a22d24950f30"
+date = "2014-11-19 22:12:50 +0000"
 branch = "None"
-tag = "glue-release-1-46"
+tag = "glue-release-1-47-1"
 if tag == "None":
     tag = None
 author = "Ryan Fisher <ryan.fisher@ligo.org>"
@@ -10,11 +10,11 @@ committer = "Ryan Fisher <ryan.fisher@ligo.org>"
 status = "CLEAN: All modifications committed"
 version = id
 verbose_msg = """Branch: None
-Tag: glue-release-1-46
-Id: 9905e10f24f54fe4f9633c6ef0d3c2b8724d8482
+Tag: glue-release-1-47-1
+Id: 45c9dd9d16bc677a761b629ab559a22d24950f30
 
 Builder: Ryan Fisher <ryan.fisher@ligo.org>
-Build date: 2013-12-03 21:30:51 +0000
+Build date: 2014-11-19 22:14:15 +0000
 Repository status: CLEAN: All modifications committed"""
 
 import warnings
@@ -33,9 +33,9 @@ def check_match(foreign_id, onmismatch="raise"):
     """
     if onmismatch not in ("raise", "warn"):
         raise ValueError, onmismatch + " is an unrecognized value of onmismatch"
-    if foreign_id == "9905e10f24f54fe4f9633c6ef0d3c2b8724d8482":
+    if foreign_id == "45c9dd9d16bc677a761b629ab559a22d24950f30":
         return
-    msg = "Program id (9905e10f24f54fe4f9633c6ef0d3c2b8724d8482) does not match given id (%s)." % foreign_id
+    msg = "Program id (45c9dd9d16bc677a761b629ab559a22d24950f30) does not match given id (%s)." % foreign_id
     if onmismatch == "raise":
         raise VersionMismatchError, msg
 
