@@ -1,4 +1,4 @@
-# Copyright (C) 2007  Kipp Cannon
+# Copyright (C) 2007-2014  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ methods that work against the SQL database.
 
 
 import itertools
+import operator
 import os
 import re
 import shutil
@@ -84,15 +85,6 @@ def connection_db_type(connection):
 	raise TypeError(connection)
 
 
-def DBTable_set_connection(connection):
-	"""
-	Set the Python DB-API 2.0 compatible connection the DBTable class
-	will use.
-	"""
-	warnings.warn("DBTable_set_connection() function is deprecated;  use a LIGOLWContentHandler subclass with a \"connection\" attribute instead.  see glue.ligolw.dbtables.use_in() for more information.", DeprecationWarning)
-	DBTable.connection = connection
-
-
 #
 # Module-level variable used to hold references to
 # tempfile.NamedTemporaryFiles objects to prevent them from being deleted
@@ -147,46 +139,48 @@ def install_signal_trap(signums = (signal.SIGTERM, signal.SIGTSTP), retval = 1):
 	Note:  this function is called by get_connection_filename()
 	whenever it creates a scratch file.
 	"""
-	with temporary_files_lock:
-		# ignore signums we've already replaced
-		signums = set(signums) - set(origactions)
+	# NOTE:  this must be called with the temporary_files_lock held.
+	# ignore signums we've already replaced
+	signums = set(signums) - set(origactions)
 
-		def temporary_file_cleanup_on_signal(signum, frame):
-			with temporary_files_lock:
-				temporary_files.clear()
-			if callable(origactions[signum]):
-				# original action is callable, chain to it
-				return origactions[signum](signum, frame)
-			# original action was not callable or the callable
-			# returned.  invoke sys.exit() with retval as exit code
-			sys.exit(retval)
+	def temporary_file_cleanup_on_signal(signum, frame):
+		with temporary_files_lock:
+			temporary_files.clear()
+		if callable(origactions[signum]):
+			# original action is callable, chain to it
+			return origactions[signum](signum, frame)
+		# original action was not callable or the callable
+		# returned.  invoke sys.exit() with retval as exit code
+		sys.exit(retval)
 
-		for signum in signums:
-			origactions[signum] = signal.getsignal(signum)
-			if origactions[signum] != signal.SIG_IGN:
-				# signal is not being ignored, so install our
-				# handler
-				signal.signal(signum, temporary_file_cleanup_on_signal)
+	for signum in signums:
+		origactions[signum] = signal.getsignal(signum)
+		if origactions[signum] != signal.SIG_IGN:
+			# signal is not being ignored, so install our
+			# handler
+			signal.signal(signum, temporary_file_cleanup_on_signal)
 
 
 def uninstall_signal_trap(signums = None):
 	"""
 	Undo the effects of install_signal_trap().  Restores the original
-	signal handlers.  If signums is a sequence of signal numbers the
-	only the signal handlers for thos signals will be restored.  If
-	signums is None (the default) then all signals that have been
-	modified by previous calls to install_signal_trap() are
-	restored.
+	signal handlers.  If signums is a sequence of signal numbers only
+	the signal handlers for those signals will be restored (KeyError
+	will be raised if one of them is not one that install_signal_trap()
+	installed a handler for, in which case some undefined number of
+	handlers will have been restored).  If signums is None (the
+	default) then all signals that have been modified by previous calls
+	to install_signal_trap() are restored.
 
 	Note:  this function is called by put_connection_filename() and
 	discard_connection_filename() whenever they remove a scratch file
 	and there are then no more scrach files in use.
 	"""
-	with temporary_files_lock:
-		if signums is None:
-			signums = origactions.keys()
-		for signum in signums:
-			signal.signal(signum, origactions.pop(signum))
+	# NOTE:  this must be called with the temporary_files_lock held.
+	if signums is None:
+		signums = origactions.keys()
+	for signum in signums:
+		signal.signal(signum, origactions.pop(signum))
 
 
 #
@@ -201,21 +195,22 @@ def get_connection_filename(filename, tmp_path = None, replace_file = False, ver
 	load.
 	"""
 	def mktmp(path, suffix = ".sqlite", verbose = False):
-		# make sure the clean-up signal traps are installed
-		install_signal_trap()
-		# create the remporary file and replace it's unlink()
-		# function
-		temporary_file = tempfile.NamedTemporaryFile(suffix = suffix, dir = path)
-		def new_unlink(self, orig_unlink = temporary_file.unlink):
-			# also remove a -journal partner, ignore all errors
-			try:
-				orig_unlink("%s-journal" % self)
-			except:
-				pass
-			orig_unlink(self)
-		temporary_file.unlink = new_unlink
-		filename = temporary_file.name
 		with temporary_files_lock:
+			# make sure the clean-up signal traps are installed
+			install_signal_trap()
+			# create the remporary file and replace it's
+			# unlink() function
+			temporary_file = tempfile.NamedTemporaryFile(suffix = suffix, dir = path if path != "_CONDOR_SCRATCH_DIR" else os.getenv("_CONDOR_SCRATCH_DIR"))
+			def new_unlink(self, orig_unlink = temporary_file.unlink):
+				# also remove a -journal partner, ignore all errors
+				try:
+					orig_unlink("%s-journal" % self)
+				except:
+					pass
+				orig_unlink(self)
+			temporary_file.unlink = new_unlink
+			filename = temporary_file.name
+			# hang onto reference to prevent its removal
 			temporary_files[filename] = temporary_file
 		if verbose:
 			print >>sys.stderr, "using '%s' as workspace" % filename
@@ -233,7 +228,7 @@ def get_connection_filename(filename, tmp_path = None, replace_file = False, ver
 			print >>sys.stderr, "'%s' exists, truncating ..." % filename,
 		try:
 			fd = os.open(filename, os.O_WRONLY | os.O_TRUNC)
-		except Exception, e:
+		except Exception as e:
 			if verbose:
 				print >>sys.stderr, "cannot truncate '%s': %s" % (filename, str(e))
 			return
@@ -254,7 +249,7 @@ def get_connection_filename(filename, tmp_path = None, replace_file = False, ver
 			# don't preserve them if the destination file
 			# already exists?
 			shutil.copystat(srcname, dstname)
-		except Exception, e:
+		except Exception as e:
 			if verbose:
 				print >>sys.stderr, "warning: ignoring failure to copy permission bits from '%s' to '%s': %s" % (filename, target, str(e))
 
@@ -277,7 +272,7 @@ def get_connection_filename(filename, tmp_path = None, replace_file = False, ver
 				while True:
 					try:
 						cpy(filename, target, verbose = verbose)
-					except IOError, e:
+					except IOError as e:
 						import errno
 						import time
 						if e.errno not in (errno.EPERM, errno.ENOSPC):
@@ -316,6 +311,8 @@ def set_temp_store_directory(connection, temp_store_directory, verbose = False):
 	"""
 	Sets the temp_store_directory parameter in sqlite.
 	"""
+	if temp_store_directory == "_CONDOR_SCRATCH_DIR":
+		temp_store_directory = os.getenv("_CONDOR_SCRATCH_DIR")
 	if verbose:
 		print >>sys.stderr, "setting the temp_store_directory to %s ..." % temp_store_directory,
 	cursor = connection.cursor()
@@ -325,31 +322,6 @@ def set_temp_store_directory(connection, temp_store_directory, verbose = False):
 		print >>sys.stderr, "done"
 
 
-#
-# FIXME:  this is only here temporarily while the file corruption issue on
-# the clusters is diagnosed.  remove when no longer needed
-#
-
-try:
-	# >= 2.5.0
-	from hashlib import md5 as __md5
-except ImportError:
-	# < 2.5.0
-	from md5 import new as __md5
-def __md5digest(filename):
-	"""
-	For internal use only.
-	"""
-	m = __md5()
-	f = open(filename)
-	while True:
-		d = f.read(4096)
-		if not d:
-			break
-		m.update(d)
-	return m.hexdigest()
-
-
 def put_connection_filename(filename, working_filename, verbose = False):
 	"""
 	This function reverses the effect of a previous call to
@@ -382,14 +354,9 @@ def put_connection_filename(filename, working_filename, verbose = False):
 		# replace document
 		if verbose:
 			print >>sys.stderr, "moving '%s' to '%s' ..." % (working_filename, filename),
-		digest_before = __md5digest(working_filename)
 		shutil.move(working_filename, filename)
-		digest_after = __md5digest(filename)
 		if verbose:
 			print >>sys.stderr, "done."
-		if digest_before != digest_after:
-			print >>sys.stderr, "md5 checksum failure!  checksum on scratch disk was %s, checksum in final location is %s" % (digest_before, digest_after)
-			sys.exit(1)
 
 		# remove reference to tempfile.TemporaryFile object.
 		# because we've just deleted the file above, this would
@@ -415,9 +382,8 @@ def put_connection_filename(filename, working_filename, verbose = False):
 		# if there are no more temporary files in place, remove the
 		# temporary-file signal traps
 		with temporary_files_lock:
-			no_more_files = not temporary_files
-		if no_more_files:
-			uninstall_signal_trap()
+			if not temporary_files:
+				uninstall_signal_trap()
 
 
 def discard_connection_filename(filename, working_filename, verbose = False):
@@ -433,20 +399,18 @@ def discard_connection_filename(filename, working_filename, verbose = False):
 	a call to get_connection_filename() even if a separate working copy
 	is not created.
 	"""
-	if working_filename != filename:
+	if working_filename == filename:
+		return
+	with temporary_files_lock:
 		if verbose:
 			print >>sys.stderr, "removing '%s' ..." % working_filename,
 		# remove reference to tempfile.TemporaryFile object
-		with temporary_files_lock:
-			del temporary_files[working_filename]
+		del temporary_files[working_filename]
 		if verbose:
 			print >>sys.stderr, "done."
-
 		# if there are no more temporary files in place, remove the
 		# temporary-file signal traps
-		with temporary_files_lock:
-			no_more_files = not temporary_files
-		if no_more_files:
+		if not temporary_files:
 			uninstall_signal_trap()
 
 
@@ -470,7 +434,7 @@ def idmap_create(connection):
 	This function is for internal use, it forms part of the code used
 	to re-map row IDs when merging multiple documents.
 	"""
-	connection.cursor().execute("CREATE TEMPORARY TABLE _idmap_ (old TEXT PRIMARY KEY, new TEXT)")
+	connection.cursor().execute("CREATE TEMPORARY TABLE _idmap_ (old TEXT PRIMARY KEY NOT NULL, new TEXT NOT NULL)")
 
 
 def idmap_reset(connection):
@@ -609,7 +573,7 @@ def get_xml(connection, table_names = None):
 		except KeyError:
 			cls = DBTable
 		table_elem = cls(AttributesImpl({u"Name": u"%s:table" % table_name}), connection = connection)
-		for column_name, column_type in get_column_info(connection, table_elem.dbtablename):
+		for column_name, column_type in get_column_info(connection, table_elem.Name):
 			if table_elem.validcolumns is not None:
 				# use the pre-defined column type
 				column_type = table_elem.validcolumns[column_name]
@@ -618,7 +582,7 @@ def get_xml(connection, table_names = None):
 				column_type = ligolwtypes.FromSQLiteType[column_type]
 			table_elem.appendChild(table.Column(AttributesImpl({u"Name": u"%s:%s" % (table_name, column_name), u"Type": column_type})))
 		table_elem._end_of_columns()
-		table_elem.appendChild(table.TableStream(AttributesImpl({u"Name": u"%s:table" % table_name})))
+		table_elem.appendChild(table.TableStream(AttributesImpl({u"Name": u"%s:table" % table_name, u"Delimiter": table.TableStream.Delimiter.default, u"Type": table.TableStream.Type.default})))
 		ligo_lw.appendChild(table_elem)
 	return ligo_lw
 
@@ -684,17 +648,6 @@ class DBTable(table.Table):
 	necessarily work with the database-backed version of the class.
 	Your mileage may vary.
 	"""
-	#
-	# When instances of this class are created, they initialize their
-	# connection attributes from the value of this class attribute.
-	#
-	# NOTE:  this is deprecated;  this class attribute will be removed,
-	# and the connection parameter of the __init__() method will be the
-	# only way to pass this information to the class in the future.
-	#
-
-	connection = None
-
 	def __new__(cls, *args, **kwargs):
 		# does this class already have table-specific metadata?
 		if not hasattr(cls, "tableName"):
@@ -733,15 +686,19 @@ class DBTable(table.Table):
 		# chain to parent class
 		table.Table.__init__(self, *args)
 
-		# save the stripped name
-		self.dbtablename = table.StripTableName(self.getAttribute(u"Name"))
-
 		# retrieve connection object from kwargs
 		self.connection = kwargs.pop("connection")
 
 		# pre-allocate a cursor for internal queries
 		self.cursor = self.connection.cursor()
 
+	def copy(self, *args, **kwargs):
+		"""
+		This method is not implemented.  See
+		glue.ligolw.table.Table for more information.
+		"""
+		raise NotImplemented
+
 	def _end_of_columns(self):
 		table.Table._end_of_columns(self)
 		# dbcolumnnames and types have the "not loaded" columns
@@ -759,8 +716,8 @@ class DBTable(table.Table):
 			"mysql": ligolwtypes.ToMySQLType
 		}[connection_db_type(self.connection)]
 		try:
-			statement = "CREATE TABLE IF NOT EXISTS " + self.dbtablename + " (" + ", ".join(map(lambda n, t: "%s %s" % (n, ToSQLType[t]), self.dbcolumnnames, self.dbcolumntypes))
-		except KeyError, e:
+			statement = "CREATE TABLE IF NOT EXISTS " + self.Name + " (" + ", ".join(map(lambda n, t: "%s %s" % (n, ToSQLType[t]), self.dbcolumnnames, self.dbcolumntypes))
+		except KeyError as e:
 			raise ValueError("column type '%s' not supported" % str(e))
 		if self.constraints is not None:
 			statement += ", " + self.constraints
@@ -775,7 +732,8 @@ class DBTable(table.Table):
 			"sqlite": ",".join("?" * len(self.dbcolumnnames)),
 			"mysql": ",".join(["%s"] * len(self.dbcolumnnames))
 		}[connection_db_type(self.connection)]
-		self.append_statement = "INSERT INTO %s (%s) VALUES (%s)" % (self.dbtablename, ",".join(self.dbcolumnnames), params)
+		self.append_statement = "INSERT INTO %s (%s) VALUES (%s)" % (self.Name, ",".join(self.dbcolumnnames), params)
+		self.append_attrgetter = operator.attrgetter(*self.dbcolumnnames)
 
 	def _end_of_rows(self):
 		# FIXME:  is this needed?
@@ -792,16 +750,16 @@ class DBTable(table.Table):
 		return self.next_id
 
 	def maxrowid(self):
-		self.cursor.execute("SELECT MAX(ROWID) FROM %s" % self.dbtablename)
+		self.cursor.execute("SELECT MAX(ROWID) FROM %s" % self.Name)
 		return self.cursor.fetchone()[0]
 
 	def __len__(self):
-		self.cursor.execute("SELECT COUNT(*) FROM %s" % self.dbtablename)
+		self.cursor.execute("SELECT COUNT(*) FROM %s" % self.Name)
 		return self.cursor.fetchone()[0]
 
 	def __iter__(self):
 		cursor = self.connection.cursor()
-		cursor.execute("SELECT * FROM %s" % self.dbtablename)
+		cursor.execute("SELECT * FROM %s" % self.Name)
 		for values in cursor:
 			yield self.row_from_cols(values)
 
@@ -810,16 +768,14 @@ class DBTable(table.Table):
 	#	# sqlite numbers rows starting from 1:  [0:10] becomes
 	#	# "rowid between 1 and 10" which means 1 <= rowid <= 10,
 	#	# which is the intended range
-	#	self.cursor.execute("DELETE FROM %s WHERE ROWID BETWEEN %d AND %d" % (self.dbtablename, i + 1, j))
+	#	self.cursor.execute("DELETE FROM %s WHERE ROWID BETWEEN %d AND %d" % (self.Name, i + 1, j))
 
 	def _append(self, row):
 		"""
 		Standard .append() method.  This method is for intended for
 		internal use only.
 		"""
-		# FIXME: in Python 2.5 use attrgetter() for attribute
-		# tuplization.
-		self.cursor.execute(self.append_statement, map(lambda n: getattr(row, n), self.dbcolumnnames))
+		self.cursor.execute(self.append_statement, self.append_attrgetter(row))
 
 	def _remapping_append(self, row):
 		"""
@@ -874,7 +830,7 @@ class DBTable(table.Table):
 			# way it will wrap is if somebody sets it to a very
 			# high number manually.  This library does not do
 			# that, so I don't bother checking.
-			self.cursor.execute("UPDATE %s SET %s WHERE ROWID > %d" % (self.dbtablename, assignments, self.last_maxrowid))
+			self.cursor.execute("UPDATE %s SET %s WHERE ROWID > %d" % (self.Name, assignments, self.last_maxrowid))
 			self.last_maxrowid = self.maxrowid() or 0
 
 
@@ -1067,7 +1023,7 @@ def use_in(ContentHandler):
 
 	>>> import sqlite3
 	>>> from glue.ligolw import ligolw
-	>>> def MyContentHandler(ligolw.LIGOLWContentHandler):
+	>>> class MyContentHandler(ligolw.LIGOLWContentHandler):
 	...	def __init__(self, *args):
 	...		super(MyContentHandler, self).__init__(*args)
 	...		self.connection = sqlite3.connection()
@@ -1078,21 +1034,14 @@ def use_in(ContentHandler):
 	Multiple database files can be in use at once by creating a content
 	handler class for each one.
 	"""
-	lsctables.use_in(ContentHandler)
+	ContentHandler = lsctables.use_in(ContentHandler)
 
 	def startTable(self, parent, attrs):
-		try:
-			connection = self.connection
-		except AttributeError:
-			warnings.warn("use of \"DBTable.connection\" class attribute to provide database connection information at DBTable instance creation time is deprecated;  use a LIGOLWContentHandler subclass with a \"connection\" attribute instead.  see glue.ligolw.dbtables.use_in() for more information.", DeprecationWarning)
-			connection = DBTable.connection
 		name = table.StripTableName(attrs[u"Name"])
 		if name in TableByName:
-			return TableByName[name](attrs, connection = connection)
-		return DBTable(attrs, connection = connection)
+			return TableByName[name](attrs, connection = self.connection)
+		return DBTable(attrs, connection = self.connection)
 
 	ContentHandler.startTable = startTable
 
-
-# FIXME:  remove
-use_in(ligolw.DefaultLIGOLWContentHandler)
+	return ContentHandler
