@@ -1,4 +1,4 @@
-# Copyright (C) 2006  Kipp Cannon
+# Copyright (C) 2006--2014  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -47,6 +47,7 @@ columns of the table.
 import copy
 import re
 import sys
+import warnings
 from xml.sax.saxutils import escape as xmlescape
 from xml.sax.xmlreader import AttributesImpl
 
@@ -79,7 +80,16 @@ __date__ = git_version.date
 # r"(?:\A[a-z0-9_]+:|\A)(?P<FullName>(?:[a-z0-9_]+:|\A)(?P<Name>[a-z0-9_]+))\Z"
 #
 # but people are putting upper case letters in names!!!!!  Someone is going
-# to get the beats.
+# to get the beats.  There is a reason for requiring names to be all lower
+# case:  SQL table and column names are case insensitive, therefore (i) in
+# general it is not possible to preserve case when documents are converted
+# into SQL databases and then converted back again, and (ii) when converted
+# to SQL the columns "Rho" and "rho" become indistinguishable and so it is
+# impossible to convert a document into an SQL database that has degenerate
+# names like this.  Requiring names to be all lower-case breaks the
+# degeneracy, helping to ensure that XML documents can be safely converted
+# to SQL databases (other rules can be imagined that would work as well,
+# this is the one that got chosen).
 
 
 ColumnPattern = re.compile(r"(?:\A\w+:|\A)(?P<FullName>(?:(?P<Table>\w+):|\A)(?P<Name>\w+))\Z")
@@ -99,6 +109,8 @@ def StripColumnName(name):
 	>>> StripColumnName("program")
 	'program'
 	"""
+	if name.lower() != name:
+		warnings.warn("column name \"%s\" is not lower case" % name)
 	try:
 		return ColumnPattern.search(name).group("Name")
 	except AttributeError:
@@ -130,7 +142,8 @@ def getColumnsByName(elem, name):
 	Return a list of Column elements named name under elem.  The name
 	comparison is done with CompareColumnNames().
 	"""
-	return elem.getElements(lambda e: (e.tagName == ligolw.Column.tagName) and (CompareColumnNames(e.getAttribute("Name"), name) == 0))
+	name = StripColumnName(name)
+	return elem.getElements(lambda e: (e.tagName == ligolw.Column.tagName) and (e.Name == name))
 
 
 #
@@ -163,6 +176,8 @@ def StripTableName(name):
 	>>> StripTableName("sngl_burst")
 	'sngl_burst'
 	"""
+	if name.lower() != name:
+		warnings.warn("table name \"%s\" is not lower case" % name)
 	try:
 		return TablePattern.search(name).group("Name")
 	except AttributeError:
@@ -190,7 +205,8 @@ def getTablesByName(elem, name):
 	Return a list of Table elements named name under elem.  The name
 	comparison is done using CompareTableNames().
 	"""
-	return elem.getElements(lambda e: (e.tagName == ligolw.Table.tagName) and (CompareTableNames(e.getAttribute("Name"), name) == 0))
+	name = StripTableName(name)
+	return elem.getElements(lambda e: (e.tagName == ligolw.Table.tagName) and (e.Name == name))
 
 
 #
@@ -204,23 +220,11 @@ def getTablesByName(elem, name):
 
 def new_from_template(template):
 	"""
-	Construct a new Table document subtree whose structure is the same
-	as the template table, that is it has the same columns etc..  The
-	rows are not copied.  Note that a fair amount of metadata is shared
-	between the original and new tables.  In particular, a copy of the
-	Table object itself is created (but with no rows), and copies of
-	the child nodes are created.  All other object references are
-	shared between the two instances, such as the RowType attribute on
-	the Table object.
+	Deprecated legacy wrapper of .copy() method of Table instances.
 	"""
-	new = copy.copy(template)
-	new.childNodes = map(copy.copy, template.childNodes)
-	for child in new.childNodes:
-		child.parentNode = new
-	del new[:]
-	new._end_of_columns()
-	new._end_of_rows()
-	return new
+	import warnings
+	warnings.warn("glue.ligolw.table.new_from_template() is deprecated.  Use .copy() method of Table instances instead.", DeprecationWarning)
+	return template.copy()
 
 
 def get_table(xmldoc, name):
@@ -235,8 +239,14 @@ def get_table(xmldoc, name):
 
 	Example:
 
-	>>> from glue.ligolw import lsctables
+	>>> import ligolw
+	>>> import lsctables
+	>>> xmldoc = ligolw.Document()
+	>>> xmldoc.appendChild(ligolw.LIGO_LW()).appendChild(lsctables.New(lsctables.SnglInspiralTable))
+	[]
 	>>> sngl_inspiral_table = get_table(xmldoc, lsctables.SnglInspiralTable.tableName)
+
+	See also the .get_table() class method of the Table class.
 	"""
 	tables = getTablesByName(xmldoc, name)
 	if len(tables) != 1:
@@ -263,6 +273,11 @@ def reassign_ids(elem):
 
 	Example:
 
+	>>> import ligolw
+	>>> import lsctables
+	>>> xmldoc = ligolw.Document()
+	>>> xmldoc.appendChild(ligolw.LIGO_LW()).appendChild(lsctables.New(lsctables.SnglInspiralTable))
+	[]
 	>>> reassign_ids(xmldoc)
 	"""
 	mapping = {}
@@ -310,9 +325,7 @@ class Column(ligolw.Column):
 	High-level column element that provides list-like access to the
 	values in a column.
 	"""
-	def __init__(self, attrs):
-		ligolw.Column.__init__(self, attrs)
-		self.asattribute = StripColumnName(self.getAttribute("Name"))
+	Name = ligolw.attributeproxy(u"Name", dec = StripColumnName)
 
 	def __len__(self):
 		"""
@@ -325,18 +338,18 @@ class Column(ligolw.Column):
 		Retrieve the value in this column in row i.
 		"""
 		if isinstance(i, slice):
-			return map(lambda r: getattr(r, self.asattribute), self.parentNode[i])
+			return map(lambda r: getattr(r, self.Name), self.parentNode[i])
 		else:
-			return getattr(self.parentNode[i], self.asattribute)
+			return getattr(self.parentNode[i], self.Name)
 
 	def __setitem__(self, i, value):
 		"""
 		Set the value in this column in row i.
 		"""
 		if isinstance(i, slice):
-			map(lambda r: setattr(r, self.asattribute, value), self.parentNode[i])
+			map(lambda r: setattr(r, self.Name, value), self.parentNode[i])
 		else:
-			setattr(self.parentNode[i], self.asattribute, value)
+			setattr(self.parentNode[i], self.Name, value)
 
 	def __iter__(self):
 		"""
@@ -344,7 +357,7 @@ class Column(ligolw.Column):
 		column.
 		"""
 		for row in self.parentNode:
-			yield getattr(row, self.asattribute)
+			yield getattr(row, self.Name)
 
 	def count(self, value):
 		"""
@@ -352,7 +365,7 @@ class Column(ligolw.Column):
 		"""
 		n = 0
 		for row in self.parentNode:
-			if getattr(row, self.asattribute) == value:
+			if getattr(row, self.Name) == value:
 				n += 1
 		return n
 
@@ -362,7 +375,7 @@ class Column(ligolw.Column):
 		equal to value.
 		"""
 		for i in xrange(len(self.parentNode)):
-			if getattr(self.parentNode[i], self.asattribute) == value:
+			if getattr(self.parentNode[i], self.Name) == value:
 				return i
 		raise ValueError(value)
 
@@ -372,7 +385,7 @@ class Column(ligolw.Column):
 		a row containing val in this column.
 		"""
 		for i in xrange(len(self.parentNode)):
-			if getattr(self.parentNode[i], self.asattribute) == value:
+			if getattr(self.parentNode[i], self.Name) == value:
 				return True
 		return False
 
@@ -382,10 +395,10 @@ class Column(ligolw.Column):
 		creates a copy of the data, so modifications made to the
 		array will *not* be recorded in the original document.
 		"""
-		if self.getAttribute("Type") not in ligolwtypes.NumericTypes:
+		if self.Type not in ligolwtypes.NumericTypes:
 			raise TypeError("Column '%s' does not have numeric type" % self.getAttribute("Name"))
 		import numpy
-		return numpy.fromiter(self, dtype = ligolwtypes.ToNumPyType[self.getAttribute("Type")])
+		return numpy.fromiter(self, dtype = ligolwtypes.ToNumPyType[self.Type])
 
 
 #
@@ -411,7 +424,26 @@ class InterningRowBuilder(tokenizer.RowBuilder):
 	results in a reduction in memory use which is small for most
 	documents, but can be subtantial when dealing with tables
 	containing large volumes of repeated information.
-	
+
+	Example:
+
+	>>> class Row(object):
+	...	pass
+	...
+	>>> # 3rd arg is optional list of attributes to intern
+	>>> rows = InterningRowBuilder(Row, ["name", "age"], ("name",))
+	>>> l = list(rows.append(["Dick", 20., "Jane", 75., "Dick", 22.]))
+	>>> l[0].name
+	'Dick'
+	>>> l[2].name
+	'Dick'
+	>>> l[2].name is l[0].name
+	True
+
+	Note that Python naturally interns short strings, so this example
+	would return True regardless;  it is intended only to demonstrate
+	the use of the class.
+
 	The values are stored in a dictionary that is shared between all
 	instances of this class, and which survives forever.  Nothing is
 	ever naturally "uninterned", so the string dictionary grows without
@@ -426,7 +458,7 @@ class InterningRowBuilder(tokenizer.RowBuilder):
 	"""
 	strings = {}
 	def append(self, tokens):
-		for row in tokenizer.RowBuilder.append(self, tokens):
+		for row in super(InterningRowBuilder, self).append(tokens):
 			for col in self.interns:
 				val = getattr(row, col)
 				setattr(row, col, self.strings.setdefault(val, val))
@@ -453,9 +485,9 @@ class TableStream(ligolw.Stream):
 	that it appends into the list-like parent element, and knows how to
 	turn the parent's rows back into a character stream.
 	"""
-	def __init__(self, attrs):
-		ligolw.Stream.__init__(self, attrs)
-		self._tokenizer = tokenizer.Tokenizer(self.getAttribute("Delimiter"))
+	def __init__(self, *args):
+		super(TableStream, self).__init__(*args)
+		self._tokenizer = tokenizer.Tokenizer(self.Delimiter)
 		self._rowbuilder = None
 
 	def config(self, parentNode):
@@ -497,7 +529,7 @@ class TableStream(ligolw.Stream):
 		# to get parsed but only if there's something other than
 		# whitespace left in the tokenizer's buffer.
 		if not self._tokenizer.data.isspace():
-			self.appendData(self.getAttribute("Delimiter"))
+			self.appendData(self.Delimiter)
 		# call parent's _end_of_rows() hook.
 		self.parentNode._end_of_rows()
 
@@ -509,7 +541,7 @@ class TableStream(ligolw.Stream):
 		# need to not put a delimiter at the end of the last row
 		# unless it ends with a null token
 		w(self.start_tag(indent))
-		rowdumper = tokenizer.RowDumper(self.parentNode.columnnames, [ligolwtypes.FormatFunc[coltype] for coltype in self.parentNode.columntypes], self.getAttribute("Delimiter"))
+		rowdumper = tokenizer.RowDumper(self.parentNode.columnnames, [ligolwtypes.FormatFunc[coltype] for coltype in self.parentNode.columntypes], self.Delimiter)
 		rowdumper.dump(self.parentNode)
 		try:
 			line = rowdumper.next()
@@ -569,15 +601,17 @@ class Table(ligolw.Table, list):
 	RowType = TableRow
 	next_id = None
 
-	def __init__(self, *attrs):
+	def __init__(self, *args):
 		"""
 		Initialize
 		"""
-		ligolw.Table.__init__(self, *attrs)
+		super(Table, self).__init__(*args)
 		self.columnnames = []
 		self.columntypes = []
 		self.columnpytypes = []
 
+	Name = ligolw.attributeproxy(u"Name", enc = (lambda name: u"%s:table" % name), dec = StripTableName)
+
 
 	#
 	# Table retrieval
@@ -597,11 +631,36 @@ class Table(ligolw.Table, list):
 
 		Example:
 
-		>>> from glue.ligolw import lsctables
+		>>> import ligolw
+		>>> import lsctables
+		>>> xmldoc = ligolw.Document()
+		>>> xmldoc.appendChild(ligolw.LIGO_LW()).appendChild(lsctables.New(lsctables.SnglInspiralTable))
+		[]
 		>>> sngl_inspiral_table = lsctables.SnglInspiralTable.get_table(xmldoc)
 		"""
 		return get_table(xmldoc, cls.tableName)
 
+	def copy(self):
+		"""
+		Construct and return a new Table document subtree whose
+		structure is the same as this table, that is it has the
+		same columns etc..  The rows are not copied.  Note that a
+		fair amount of metadata is shared between the original and
+		new tables.  In particular, a copy of the Table object
+		itself is created (but with no rows), and copies of the
+		child nodes are created.  All other object references are
+		shared between the two instances, such as the RowType
+		attribute on the Table object.
+		"""
+		new = copy.copy(self)
+		new.childNodes = map(copy.copy, self.childNodes)
+		for child in new.childNodes:
+			child.parentNode = new
+		del new[:]
+		new._end_of_columns()
+		new._end_of_rows()
+		return new
+
 
 	#
 	# Column access
@@ -615,6 +674,8 @@ class Table(ligolw.Table, list):
 
 		Example:
 
+		>>> import lsctables
+		>>> tbl = lsctables.New(lsctables.SnglInspiralTable)
 		>>> col = tbl.getColumnByName("mass1")
 		"""
 		try:
@@ -642,10 +703,12 @@ class Table(ligolw.Table, list):
 		Example:
 
 		>>> import lsctables
-		>>> process_table = get_table(xmldoc, lsctabes.Process.tableName)
+		>>> process_table = lsctables.New(lsctables.ProcessTable, [])
 		>>> col = process_table.appendColumn("program")
 		>>> col.getAttribute("Name")
 		'process:program'
+		>>> col.Name
+		'program'
 		"""
 		try:
 			self.getColumnByName(name)
@@ -675,22 +738,20 @@ class Table(ligolw.Table, list):
 		del self.columntypes[:]
 		del self.columnpytypes[:]
 		for child in self.getElementsByTagName(ligolw.Column.tagName):
-			colname = StripColumnName(child.getAttribute("Name"))
-			llwtype = child.getAttribute("Type")
 			if self.validcolumns is not None:
 				try:
-					if self.validcolumns[colname] != llwtype:
-						raise ligolw.ElementError("invalid type '%s' for Column '%s' in Table '%s', expected type '%s'" % (llwtype, child.getAttribute("Name"), self.getAttribute("Name"), self.validcolumns[colname]))
+					if self.validcolumns[child.Name] != child.Type:
+						raise ligolw.ElementError("invalid type '%s' for Column '%s' in Table '%s', expected type '%s'" % (child.Type, child.getAttribute("Name"), self.getAttribute("Name"), self.validcolumns[child.Name]))
 				except KeyError:
 					raise ligolw.ElementError("invalid Column '%s' for Table '%s'" % (child.getAttribute("Name"), self.getAttribute("Name")))
-			if colname in self.columnnames:
+			if child.Name in self.columnnames:
 				raise ligolw.ElementError("duplicate Column '%s' in Table '%s'" % (child.getAttribute("Name"), self.getAttribute("Name")))
-			self.columnnames.append(colname)
-			self.columntypes.append(llwtype)
+			self.columnnames.append(child.Name)
+			self.columntypes.append(child.Type)
 			try:
-				self.columnpytypes.append(ligolwtypes.ToPyType[llwtype])
+				self.columnpytypes.append(ligolwtypes.ToPyType[child.Type])
 			except KeyError:
-				raise ligolw.ElementError("unrecognized Type '%s' for Column '%s' in Table '%s'" % (llwtype, child.getAttribute("Name"), self.getAttribute("Name")))
+				raise ligolw.ElementError("unrecognized Type '%s' for Column '%s' in Table '%s'" % (child.Type, child.getAttribute("Name"), self.getAttribute("Name")))
 
 	def _verifyChildren(self, i):
 		"""
@@ -702,6 +763,7 @@ class Table(ligolw.Table, list):
 		if child.tagName == ligolw.Column.tagName:
 			self._update_column_info()
 		elif child.tagName == ligolw.Stream.tagName:
+			# require agreement of non-stripped strings
 			if child.getAttribute("Name") != self.getAttribute("Name"):
 				raise ligolw.ElementError("Stream name '%s' does not match Table name '%s'" % (child.getAttribute("Name"), self.getAttribute("Name")))
 
@@ -728,7 +790,7 @@ class Table(ligolw.Table, list):
 		Remove a child from this element.  The child element is
 		returned, and it's parentNode element is reset.
 		"""
-		ligolw.Table.removeChild(self, child)
+		super(Table, self).removeChild(child)
 		if child.tagName == ligolw.Column.tagName:
 			self._update_column_info()
 		return child
@@ -738,7 +800,7 @@ class Table(ligolw.Table, list):
 		Break internal references within the document tree rooted
 		on this element to promote garbage collection.
 		"""
-		ligolw.Table.unlink(self)
+		super(Table, self).unlink()
 		del self[:]
 
 	def endElement(self):
@@ -753,6 +815,7 @@ class Table(ligolw.Table, list):
 	# Row ID manipulation
 	#
 
+	@classmethod
 	def get_next_id(cls):
 		"""
 		Returns the current value of the next_id class attribute,
@@ -763,8 +826,8 @@ class Table(ligolw.Table, list):
 		id = cls.next_id
 		cls.next_id += 1
 		return id
-	get_next_id = classmethod(get_next_id)
 
+	@classmethod
 	def set_next_id(cls, id):
 		"""
 		Sets the value of the next_id class attribute.  This is a
@@ -773,32 +836,31 @@ class Table(ligolw.Table, list):
 		attribute.
 		"""
 		cls.next_id = id
-	set_next_id = classmethod(set_next_id)
 
 	def sync_next_id(self):
 		"""
 		Determines the highest-numbered ID in this table, and sets
-		the table's next_id attribute to the next highest ID in
-		sequence.  If the next_id attribute is already set to a
-		value greater than the max found, then it is left
+		the table's .next_id attribute to the next highest ID in
+		sequence.  If the .next_id attribute is already set to a
+		value greater than the highest value found, then it is left
 		unmodified.  The return value is the ID identified by this
-		method.  If the table's next_id attribute is None, then
+		method.  If the table's .next_id attribute is None, then
 		this function is a no-op.
 
 		Note that tables of the same name typically share a common
-		next_id attribute (it is a class attribute, not an
-		attribute of each instance).  This is enforced so that IDs
-		can be generated that are unique across all tables in the
-		document.  Running sync_next_id() on all the tables in a
-		document that are of the same type will have the effect of
-		setting the ID to the next ID higher than any ID in any of
-		those tables.
+		.next_id attribute (it is a class attribute, not an
+		attribute of each instance) so that IDs can be generated
+		that are unique across all tables in the document.  Running
+		sync_next_id() on all the tables in a document that are of
+		the same type will have the effect of setting the ID to the
+		next ID higher than any ID in any of those tables.
 
 		Example:
 
-		>>> for tbl in xmldoc.getElementsByTagName(Table.tagName):
-		...	tbl.sync_next_id()
-		...
+		>>> import lsctables
+		>>> tbl = lsctables.New(lsctables.ProcessTable)
+		>>> print tbl.sync_next_id()
+		process:process_id:0
 		"""
 		if self.next_id is not None:
 			if len(self):
@@ -826,6 +888,8 @@ class Table(ligolw.Table, list):
 			# table is missing its ID column, this is a no-op
 			return mapping
 		for i, old in enumerate(column):
+			if old is None:
+				raise ValueError("null row ID encountered in Table '%s', row %d" % (self.getAttribute("Name"), i))
 			if old in mapping:
 				column[i] = mapping[old]
 			else:
@@ -872,11 +936,12 @@ def use_in(ContentHandler):
 	Example:
 
 	>>> from glue.ligolw import ligolw
-	>>> def MyContentHandler(ligolw.LIGOLWContentHandler):
+	>>> class LIGOLWContentHandler(ligolw.LIGOLWContentHandler):
 	...	pass
 	...
 	>>> from glue.ligolw import table
-	>>> table.use_in(MyContentHandler)
+	>>> table.use_in(LIGOLWContentHandler)
+	<class 'glue.ligolw.table.LIGOLWContentHandler'>
 	"""
 	def startColumn(self, parent, attrs):
 		return Column(attrs)
@@ -894,6 +959,4 @@ def use_in(ContentHandler):
 	ContentHandler.startStream = startStream
 	ContentHandler.startTable = startTable
 
-
-# FIXME:  remove
-use_in(ligolw.DefaultLIGOLWContentHandler)
+	return ContentHandler
