@@ -713,7 +713,7 @@ PyTypeObject ligolw_Tokenizer_Type = {
 "Example:\n" \
 "\n" \
 ">>> from glue.ligolw import tokenizer\n" \
-">>> t = tokenizer.Tokenizer(\",\")\n" \
+">>> t = tokenizer.Tokenizer(u\",\")\n" \
 ">>> t.set_types([str, int])\n" \
 ">>> list(t.append(\"a,10,b,2\"))\n" \
 "['a', 10, 'b']\n" \
